// src/index.ts
import { createUnplugin } from "unplugin";
import { createFilter, normalizePath } from "@rollup/pluginutils";
import {
  REGEX_SETUP_SFC,
  REGEX_SRC_FILE,
  REGEX_VUE_SFC,
  REGEX_VUE_SUB
} from "@vue-macros/common";
import { shouldTransform as shouldTransform2, transform } from "@vue/reactivity-transform";

// src/core/index.ts
import {
  DEFINE_PROPS,
  MagicString,
  babelParse,
  getTransformResult,
  isCallOf,
  parseSFC,
  resolveObjectKey
} from "@vue-macros/common";
import { shouldTransform, transformAST } from "@vue/reactivity-transform";

// src/core/helper.ts
var helperId = "/vue2-reactivity-transform-helper";
var helperCode = `export function createPropsRestProxy(props, excludedKeys) {
  const ret = {}
  for (const key in props) {
    if (!excludedKeys.includes(key)) {
      Object.defineProperty(ret, key, {
        enumerable: true,
        get: () => props[key],
      })
    }
  }
  return ret
}`;

// src/core/index.ts
function transfromVueSFC(code, id) {
  const s = new MagicString(code);
  const { script, scriptSetup } = parseSFC(code, id);
  let refBindings;
  let propsDestructuredBindings;
  if (script && shouldTransform(script.content)) {
    const ast = babelParse(script.content, script.lang);
    const offset = script.loc.start.offset;
    const { importedHelpers, rootRefs } = transformAST(ast, s, offset);
    refBindings = rootRefs;
    addHelpers(s, script.loc.start.offset, importedHelpers);
  }
  if (scriptSetup) {
    const ast = babelParse(scriptSetup.content, scriptSetup.lang);
    for (const node of ast.body) {
      processDefineProps(node);
    }
    if (propsDestructuredBindings || refBindings || shouldTransform(scriptSetup.content)) {
      const { importedHelpers } = transformAST(
        ast,
        s,
        scriptSetup.loc.start.offset,
        refBindings,
        propsDestructuredBindings
      );
      addHelpers(s, scriptSetup.loc.start.offset, importedHelpers);
    }
  }
  return getTransformResult(s, id);
  function processDefineProps(node) {
    if (node.type !== "VariableDeclaration")
      return;
    const decl = node.declarations.find(
      (decl2) => isCallOf(decl2.init, DEFINE_PROPS)
    );
    if (!decl || decl.id.type !== "ObjectPattern")
      return;
    if (node.declarations.length > 1)
      throw new SyntaxError(
        `${DEFINE_PROPS}() don't support multiple declarations.`
      );
    const offset = scriptSetup.loc.start.offset;
    let defaultStr = "";
    propsDestructuredBindings = {};
    for (const prop of decl.id.properties) {
      if (prop.type === "ObjectProperty") {
        const propKey = resolveObjectKey(prop.key, prop.computed, false);
        if (!propKey) {
          throw new SyntaxError(
            `${DEFINE_PROPS}() destructure cannot use computed key.`
          );
        }
        if (prop.value.type === "AssignmentPattern") {
          const { left, right } = prop.value;
          if (left.type !== "Identifier") {
            throw new SyntaxError(
              `${DEFINE_PROPS}() destructure does not support nested patterns.`
            );
          }
          propsDestructuredBindings[propKey] = {
            local: left.name
          };
          defaultStr += `${propKey}: ${s.sliceNode(right, { offset })},`;
        } else if (prop.value.type === "Identifier") {
          propsDestructuredBindings[propKey] = {
            local: prop.value.name
          };
        } else {
          throw new SyntaxError(
            `${DEFINE_PROPS}() destructure does not support nested patterns.`
          );
        }
      } else {
        s.prependLeft(
          offset,
          `import { createPropsRestProxy } from '${helperId}';
const ${prop.argument.name} = createPropsRestProxy(__props, ${JSON.stringify(
            Object.keys(propsDestructuredBindings)
          )});
`
        );
      }
    }
    const declStr = `withDefaults(${s.sliceNode(decl.init, {
      offset
    })}, { ${defaultStr} })`;
    s.overwriteNode(node, declStr, { offset });
  }
}
function addHelpers(s, offset, helpers) {
  if (helpers.length === 0)
    return;
  s.prependLeft(
    offset,
    `import { ${helpers.map((h) => `${h} as _${h}`).join(", ")} } from 'vue';
`
  );
}

// src/index.ts
function resolveOption(options, framework) {
  return {
    include: [REGEX_SRC_FILE, REGEX_VUE_SFC, REGEX_SETUP_SFC].concat(
      framework === "webpack" ? REGEX_VUE_SUB : []
    ),
    exclude: [/node_modules/],
    ...options
  };
}
var name = "unplugin-reactivity-transform-vue2";
var src_default = createUnplugin(
  (userOptions = {}, { framework }) => {
    const options = resolveOption(userOptions, framework);
    const filter = createFilter(options.include, options.exclude);
    return {
      name,
      enforce: "pre",
      resolveId(id) {
        if (id === helperId)
          return id;
      },
      loadInclude(id) {
        return id === helperId;
      },
      load(_id) {
        const id = normalizePath(_id);
        if (id === helperId)
          return helperCode;
      },
      transformInclude(id) {
        return filter(id);
      },
      transform(code, id) {
        try {
          if (REGEX_VUE_SFC.test(id) || REGEX_SETUP_SFC.test(id) || framework === "webpack" && REGEX_VUE_SUB.test(id)) {
            return transfromVueSFC(code, id);
          } else {
            if (!shouldTransform2(code))
              return;
            return transform(code, {
              filename: id,
              sourceMap: true
            });
          }
        } catch (err) {
          this.error(`${name} ${err}`);
        }
      }
    };
  }
);

export {
  src_default
};
